@@ -11,7 +11,7 @@
  *  You may distribute this under the terms of either the GNU General Public
  *  License or the Artistic License, as specified in the Perl README file.
  *
- *  $Id: dbdimp.h,v 1.11 2004/10/04 00:50:08 rlippan Exp $
+ *  $Id: dbdimp.h,v 1.15 2005/04/02 06:50:05 rlippan Exp $
  */
 
 /*
@@ -146,7 +146,7 @@ struct imp_sth_st {
 
     MYSQL_RES* cda;       /* result                                 */
     int currow;           /* number of current row                  */
-    long row_num;         /* total number of rows                   */
+    my_ulonglong row_num;         /* total number of rows                   */
 
     int   done_desc;      /* have we described this sth yet ?	    */
     long  long_buflen;    /* length for long/longraw (if >0)	    */
@@ -163,7 +163,8 @@ struct imp_sth_st {
 /*
  *  And last, not least: The prototype definitions.
  *
- * These defines avoid name clashes for multiple statically linked DBD's	*/
+ * These defines avoid name clashes for multiple statically linked DBD's
+ */
 #define dbd_init		mysql_dr_init
 #define dbd_db_login		mysql_db_login
 #define dbd_db_do		mysql_db_do
@@ -190,14 +191,19 @@ struct imp_sth_st {
 #define do_error		mysql_dr_error
 #define dbd_db_type_info_all    mysql_db_type_info_all
 #define dbd_db_quote            mysql_db_quote
+
+#ifdef DBD_MYSQL_INSERT_ID_IS_GOOD /* prototype was broken in 
+				      some versions of dbi */
 #define dbd_db_last_insert_id   mysql_db_last_insert_id
 
+#endif
+
 #include <dbd_xsh.h>
 void	 do_error (SV* h, int rc, const char *what);
 SV	*dbd_db_fieldlist (MYSQL_RES* res);
 
 void    dbd_preparse (imp_sth_t *imp_sth, SV *statement);
-int mysql_st_internal_execute(SV*, SV*, SV*, int, imp_sth_ph_t*, MYSQL_RES**,
+my_ulonglong mysql_st_internal_execute(SV*, SV*, SV*, int, imp_sth_ph_t*, MYSQL_RES**,
 			      MYSQL*, int);
 AV* dbd_db_type_info_all (SV* dbh, imp_dbh_t* imp_dbh);
 SV* dbd_db_quote(SV*, SV*, SV*);
@@ -207,6 +213,5 @@ extern MYSQL* mysql_dr_connect(MYSQL*, char*, char*, char*, char*, char*,
 
 extern int mysql_db_reconnect(SV*);
 
-SV *mysql_db_last_insert_id(SV* dbh, imp_dbh_t *imp_dbh,
-        SV *catalog, SV *schema, SV *table, SV *field,SV *attr);
-
+/* SV *mysql_db_last_insert_id(SV* dbh, imp_dbh_t *imp_dbh,
+        SV *catalog, SV *schema, SV *table, SV *field,SV *attr); */
